// tgStrList by Tom Grandgent - copyright (c) 2003
//
// Class for managing a list of strings and associated information (key/value pairs) with each string
// (this is basically a dictionary now)
//
// Note: *Removing* strings, keys, or values is not implemented.  Only add/set/get.

#include "tgStrList.h"

// Returns the linked list node for the given string
strlist_t *tgStrList::findString(char *str)
{
	strlist_t *p = root;
	while (p)
	{
		if (!stricmp(str, p->str))
			return p;
		p = p->next;
	}
	return NULL;
}

// Adds the given string to the given linked list
strlist_t *tgStrList::addNode(strlist_t **rootNode, strlist_t **tailNode, char *newStr)
{
	strlist_t *p = (strlist_t *) malloc(sizeof(strlist_t));
	p->str = strdup(newStr);
	p->data = p->next = p->subList = p->subListTail = NULL;
	if (!*tailNode)
		*rootNode = *tailNode = p;
	else
	{
		(*tailNode)->next = p;
		*tailNode = p;
	}

	return p;
}

// Constructor
tgStrList::tgStrList()
{
	root = tail = nextItem = NULL;
	itemCount = 0;
}

// Add a string to the list
void tgStrList::add(char *newStr)
{
	if (!newStr)
		return;

	addNode(&root, &tail, newStr);
	itemCount++;
}

// Reset enumeration
void tgStrList::reset(void)
{
	nextItem = root;
}

// Return next string in enumeration
char *tgStrList::next(void)
{
	if (nextItem)
	{
		char *s = nextItem->str;
		nextItem = nextItem->next;
		return s;
	}
	else
		return NULL;
}

// Gets data for a particular string
void *tgStrList::getValue(char *str, char *key)
{
	// Find the matching string
	strlist_t *p = findString(str);
	if (!p)
		return NULL;

	// Find the matching key
	strlist_t *q = p->subList;
	while (q)
	{
		if (!stricmp(q->str, key))
			return q->data;
		q = q->next;
	}
	return NULL;
}

// Gets an int for a particular string
int tgStrList::getValueInt(char *str, char *key)
{
	void *p = getValue(str, key);
	if (p)
		return (int) p;
	return 0;
}

// Gets a string for a particular string
char *tgStrList::getValueString(char *str, char *key)
{
	void *p = getValue(str, key);
	if (p)
		return (char *) p;
	return NULL;
}

// Sets data for a particular string - returns 0 on success, 1 on error
int tgStrList::setValue(char *str, char *key, void *val)
{
	// Find the matching string
	strlist_t *p = findString(str);
	if (!p)
		return 1;

	// Find the matching key
	strlist_t *q = p->subList;
	while (q)
	{
		if (!stricmp(q->str, key))
			break;
		q = q->next;
	}

	// Couldn't find the key?  Create it.
	if (!q)
		q = addNode(&(p->subList), &(p->subListTail), key);

	// Set the data
	q->data = val;

	return 0;
}

// Sets an int for a particular string - returns 0 on success, 1 on error
int tgStrList::setValueInt(char *str, char *key, int val)
{
	return setValue(str, key, (void *) val);
}

// Sets a string for a particular string - returns 0 on success, 1 on error
int tgStrList::setValueString(char *str, char *key, char *val)
{
	if (!val)
		return 1;

	char *ch = strdup(val);
	int err = setValue(str, key, (void *) val);
	if (err)
		free(val);
	return err;
}